package be.iminds.ilabt.jfed.ui.cli2.instruction;

import be.iminds.ilabt.jfed.experiment.setup.ExperimentSetupConfig;
import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.annotation.JsonTypeInfo;
import com.fasterxml.jackson.databind.annotation.JsonTypeIdResolver;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.util.List;

@JsonTypeInfo(
        use = JsonTypeInfo.Id.CUSTOM,
        include = JsonTypeInfo.As.PROPERTY,
        property = "action",
        defaultImpl = JsonTypeInfo.class, //no default!
        visible = true)
@JsonTypeIdResolver(InstructionTypeIdResolver.class)
public abstract class InstructionWithSlice extends Instruction {
    @Nonnull protected final ExperimentSetupConfig.Slice slice;

    @JsonCreator
    public InstructionWithSlice(@JsonProperty("action") @Nonnull Action action,
                                @JsonProperty("user") @Nullable User user,
                                @JsonProperty("actionOutputs") @Nullable List<ActionOutput> actionOutputs,
                                @JsonProperty("debugOutputs") @Nullable List<DebugOutput> debugOutputs,
                                @JsonProperty("callOutputs") @Nullable List<CallOutput> callOutputs,
                                @JsonProperty("slice") ExperimentSetupConfig.Slice slice) {
        super(action, user, actionOutputs, debugOutputs, callOutputs);
        this.slice = slice == null ? new ExperimentSetupConfig.Slice() : slice;
    }

    protected InstructionWithSlice(@Nonnull Action action, boolean demo, boolean sliceToRecover) {
        super(action, demo);
        if (demo) {
            if (sliceToRecover) {
                this.slice =
                        new ExperimentSetupConfig.Slice("my_slice_name",
                                null, null, null, null,
                                null, null, null);
            } else {
                this.slice =
                        new ExperimentSetupConfig.Slice("my_slice_name",
                                false, false, 120L, true,
                                ExperimentSetupConfig.ProjectSource.PROVIDED, true, "my_project_s_name");
            }
        } else {
            this.slice = new ExperimentSetupConfig.Slice();
        }
    }

    @JsonProperty
    @Nonnull
    public ExperimentSetupConfig.Slice getSlice() {
        return slice;
    }
}
