package be.iminds.ilabt.jfed.experimenter_gui.dialogs;

import be.iminds.ilabt.jfed.experimenter_gui.events.OpenRequestRspecStringEvent;
import be.iminds.ilabt.jfed.fedmon.webapi.service.json.Testbed;
import be.iminds.ilabt.jfed.lowlevel.user.GeniUserProvider;
import be.iminds.ilabt.jfed.lowlevel.connection_pool.SfaConnectionPool;
import be.iminds.ilabt.jfed.preferences.JFedPreferences;
import be.iminds.ilabt.jfed.rspec.model.*;
import be.iminds.ilabt.jfed.rspec.rspec_source.RequestRspecSource;
import be.iminds.ilabt.jfed.util.library.JFedTrustStore;
import com.google.common.eventbus.EventBus;
import javafx.application.Platform;
import javafx.beans.value.ObservableValue;
import javafx.concurrent.Worker;
import javafx.fxml.FXML;
import javafx.stage.Window;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.annotation.Nonnull;
import javax.inject.Inject;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

public class TestbedSiteDialog extends JFedBrowser {
    private static final Logger LOG = LoggerFactory.getLogger(TestbedSiteDialog.class);

    private Testbed testbed;
    private String url;
    private String title;

    @Nonnull private final EventBus eventBus;

    public static class TestbedSiteDialogJavaScriptBridge extends JavaScriptBridge {
        @Nonnull private final TestbedSiteDialog testbedSiteDialog;
        public TestbedSiteDialogJavaScriptBridge(@Nonnull TestbedSiteDialog testbedSiteDialog) {
            super(testbedSiteDialog);
            this.testbedSiteDialog = testbedSiteDialog;
        }

        final List<String> nodeUrns = new ArrayList<>();
        final List<String> requestRspecs = new ArrayList<>();
        String requestRspec = null;

        public void addRequestRspec(String requestRSpec) {
            requestRspecs.add(requestRSpec);
        }

        public void setRequestRspec(String requestRSpec) {
            requestRspecs.clear();
            nodeUrns.clear();
            requestRspec = requestRSpec;
        }

        public void addNode(String urn) {
            nodeUrns.add(urn);
        }

        @Override
        public void close() {
            LOG.debug("TestbedSiteDialogJavaScriptBridge.close");

            RequestRspecSource requestRspecSource = requestRspec == null ? null :
                    new RequestRspecSource(requestRspec, ModelRspecType.FX);
            ModelRspec tmpModelRspec = null;
            if (requestRspecSource != null) {
                tmpModelRspec = requestRspecSource.getModelRspec();
            }
            RspecFactory factory = RspecFactoryFactory.getRspecFactoryInstance(ModelRspecType.FX);
            if (tmpModelRspec == null) {
                tmpModelRspec = RspecFactoryFactory.getRspecFactoryInstance(ModelRspecType.FX)
                        .createModelRspec("request");
            }
            ModelRspec modelRspec = tmpModelRspec;

            for (String req : requestRspecs) {
                RequestRspecSource addedRRS = new RequestRspecSource(req, ModelRspecType.FX);
                ModelRspec addedMR = addedRRS.getModelRspec();
                if (addedMR == null) continue;
                addedMR.getNodes().forEach(modelRspec::addNode);
            }

            for (String nodeUrn : nodeUrns) {
                RspecNode node = factory.createNode(modelRspec);
                node.setComponentManagerId(testbedSiteDialog.testbed.getDefaultComponentManagerUrn());
                node.setComponentId(nodeUrn);
                node.setExclusive(true);
                node.setSliverTypeName("raw-pc");
                modelRspec.addNode(node);
            }

            testbedSiteDialog.eventBus.post(new OpenRequestRspecStringEvent(modelRspec.toGeni3Rspec()));

            Window stage = testbedSiteDialog.scene.getWindow();
            stage.hide();
        }
    }

    @Inject
    public TestbedSiteDialog(@Nonnull GeniUserProvider geniUserProvider,
                             @Nonnull SfaConnectionPool connectionPool,
                             @Nonnull JFedTrustStore trustStore,
                             @Nonnull JFedPreferences preferences,
                             @Nonnull EventBus eventBus) {
        super(geniUserProvider, connectionPool, trustStore, preferences);
        this.eventBus = eventBus;
    }

    public void setUrlAndTestbed(@Nonnull URL url,
                                 @Nonnull Testbed testbed,
                                 @Nonnull String title) {
        this.url = url.toExternalForm();
        this.testbed = testbed;
        this.title = title;

        loadPageWhenReady();
    }

    @FXML
    @Override
    protected void initialize() {
        webEngine = webView.getEngine();

        URLStreamFactoryCustomizer.useDedicatedProxyForWebkit(
                new MyHttpUrlStreamHandler(), "https");
        initialized = true;

        webEngine.getLoadWorker().stateProperty().addListener(
                (ObservableValue<? extends Worker.State> ov, Worker.State oldState,
                 Worker.State newState) -> {
                    if (newState == Worker.State.SUCCEEDED) {
                        jsBridgeBackoff = 1;
                        registerJsBridge();
                    }
                });
        loadPageWhenReady();
    }

    @Override
    protected void loadPageWhenReady() {
        if (!initialized)
            return;

        if (this.testbed == null)
            return;

        Platform.runLater(this::registerJsBridge);
        loadPage();
    }

    @Override
    protected Object makeJavascriptBridge() {
        return new TestbedSiteDialogJavaScriptBridge(this);
    }

    protected void loadPage() {
        headerLabel.setText(title);
        webEngine.load(url);
    }

}
